# -*- coding: utf-8 -*-


"""distro.distro: provides entry point main()."""
from __future__ import absolute_import

# import sys
import argparse
from pkg_resources import get_distribution

from distro.data import DATA


def main():
    """Main execution wrapper. Parsers CLI arguments, loads data, calls the search method and displays the results"""
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--output', type=str, default='text', help='Output format (text|json|csv)')
    parser.add_argument('-v', '--version', action='store_true', help='Show version')
    parser.add_argument('search_term', nargs='*', help='Search terms, it can be a string or field:string')
    args = parser.parse_args()

    if args.version:
        version = get_distribution('distro-names').version
        print(f'v{version}')
        return

    distros = load()
    results = search(args.search_term, distros)

    if args.output == 'text':
        display_text(results)
    elif args.output == 'json':
        display_json(results)
    elif args.output == 'csv':
        display_csv(results)


def load():
    """Load data from distro/data.py, return a list of Distro() objects"""
    return [Distro(**dt) for dt in DATA]


def search(terms, distros):
    """Search for items in the list of Distro() objects using the provided search terms."""
    results = []

    for dist in distros:
        if all(t in dist.raw() for t in terms):
            results.append(dist)

    return results


def display_text(results):
    """Display search results in raw TEXT format"""
    for result in results:
        print(result)


def display_csv(results):
    """Display search results in CSV format"""
    for result in results:
        print(result.raw())


def display_json(results):
    """Display search results in JSON format"""
    print('NOT IMPLEMENTED')
    for result in results:
        print(result.raw())


class Distro:
    """Distro data wrapper class. Instantiate with a dict"""

    def __init__(self, **kwargs):
        self.data = kwargs

        if 'name' not in self.data.keys():
            raise Exception('Name is required')
        if 'version' not in self.data.keys() and 'versions' not in self.data.keys():
            raise Exception('Either version or list of versions required')
        if 'flavour' not in self.data.keys():
            self.data['flavour'] = 'Linux'

        if self.data.get('versions', None) is None:
            self.data['versions'] = list()
        elif isinstance(self.data.get('versions', None), str):
            self.data['versions'] = list(self.data['versions'])

        ver = self.data.pop('version', None)
        if ver is not None and ver not in self.data['versions']:
            self.data['versions'].append(ver)

    def __str__(self):
        lpad = max([len(key) for key, val in self.data.items()]) + 5
        out_str = ''
        for key, val in sorted(self.data.items()):
            out_str += '{}:'.format(key.replace('_', ' ').title()).ljust(lpad, ' ')
            if isinstance(val, list):
                out_str += '{}\n'.format(', '.join(val))
            else:
                out_str += '{}\n'.format(val)
        return out_str

    def raw(self):
        """Return data in simplified CSV format, all in lowercase"""
        out = list()
        for key, val in sorted(self.data.items()):
            if isinstance(val, list):
                out.append('{}:{}'.format(key, ','.join(val)))
            else:
                out.append('{}:{}'.format(key, val))
        return ','.join(out).lower()



"""
The Distro data. Names, versions, and any other information.

The DATA variable is a Python LIST of DICTs, each dict is required, at a minimum:

- a 'name' key of type String
- either a 'version' key of type String OR a 'versions' key of type List (of strings)

If no 'flavour' key is provided, the default will be 'Linux'

Any other key/value pair will be added as is to the Distro object
"""

DATA = [
    # Ubuntu (source: http://releases.ubuntu.com/)
    {'name': 'precise', 'versions': ['12.04', '12.04.5'], 'full_name': 'Precise Pangolin', 'flavour': 'Ubuntu'},
    {'name': 'trusty', 'versions': ['14.04', '14.04.6'], 'full_name': 'Trusty Thar', 'flavour': 'Ubuntu'},
    {'name': 'xenial', 'versions': ['16.04', '16.04.6'], 'full_name': 'Xenial Xerus', 'flavour': 'Ubuntu'},
    {'name': 'bionic', 'versions': ['18.04', '18.04.2'], 'full_name': 'Bionic Beaver', 'flavour': 'Ubuntu'},
    {'name': 'cosmic', 'versions': ['18.10'], 'full_name': 'Cosmic Cuttlefish', 'flavour': 'Ubuntu'},
    {'name': 'disco', 'versions': ['19.04'], 'full_name': 'Disco Dingo', 'flavour': 'Ubuntu'},
    {'name': 'fossa', 'versions': ['20.04'], 'full_name': 'Focal Fossa', 'flavour': 'Ubuntu'},
    {'name': 'gorilla', 'versions': ['20.10'], 'full_name': 'Groovy Gorilla', 'flavour': 'Ubuntu'},

    # Debian (source: https://www.debian.org/releases/)
    {'name': 'squeeze', 'version': '6', 'flavour': 'Debian'},
    {'name': 'wheezy', 'version': '7', 'flavour': 'Debian'},
    {'name': 'jessie', 'version': '8', 'flavour': 'Debian'},
    {'name': 'stretch', 'version': '9', 'flavour': 'Debian'},
    {'name': 'buster', 'version': '10', 'flavour': 'Debian'},
    {'name': 'bullseye', 'version': '11', 'flavour': 'Debian'},

    # Mint (source: https://en.wikipedia.org/wiki/Linux_Mint_version_history)
    {'name': 'ada', 'version': '1.0', 'flavour': 'Mint'},
    {'name': 'barbara', 'version': '2.0', 'flavour': 'Mint'},
    {'name': 'bea', 'version': '2.1', 'flavour': 'Mint'},
    {'name': 'bianca', 'version': '2.2', 'flavour': 'Mint'},
    {'name': 'cassandra', 'version': '3.0', 'flavour': 'Mint'},
    {'name': 'celena', 'version': '3.1', 'flavour': 'Mint'},
    {'name': 'daryna', 'version': '4', 'flavour': 'Mint'},
    {'name': 'elyssa', 'version': '5', 'flavour': 'Mint'},
    {'name': 'felicia', 'version': '6', 'flavour': 'Mint'},
    {'name': 'gloria', 'version': '7', 'flavour': 'Mint'},
    {'name': 'helena', 'version': '8', 'flavour': 'Mint'},
    {'name': 'isadora', 'version': '9', 'flavour': 'Mint'},
    {'name': 'julia', 'version': '10', 'flavour': 'Mint'},
    {'name': 'katya', 'version': '11', 'flavour': 'Mint'},
    {'name': 'lisa', 'version': '12', 'flavour': 'Mint'},
    {'name': 'maya', 'version': '13', 'flavour': 'Mint'},
    {'name': 'nadia', 'version': '14', 'flavour': 'Mint'},
    {'name': 'olivia', 'version': '15', 'flavour': 'Mint'},
    {'name': 'petra', 'version': '16', 'flavour': 'Mint'},
    {'name': 'qiana', 'version': '17', 'flavour': 'Mint'},
    {'name': 'rebecca', 'version': '17.1', 'flavour': 'Mint'},
    {'name': 'rafaela', 'version': '17.2', 'flavour': 'Mint'},
    {'name': 'rosa', 'version': '17.3', 'flavour': 'Mint'},
    {'name': 'sarah', 'version': '18', 'flavour': 'Mint'},
    {'name': 'serena', 'version': '18.1', 'flavour': 'Mint'},
    {'name': 'sonya', 'version': '18.2', 'flavour': 'Mint'},
    {'name': 'sylvia', 'version': '18.3', 'flavour': 'Mint'},
    {'name': 'tara', 'version': '19', 'flavour': 'Mint'},
    {'name': 'tessa', 'version': '19.1', 'flavour': 'Mint'},
    {'name': 'tina', 'version': '19.2', 'flavour': 'Mint'},
    {'name': 'tricia', 'version': '19.3', 'flavour': 'Mint'},
    {'name': 'ulyana', 'version': '20', 'flavour': 'Mint'},
    {'name': 'ulyssa', 'version': '20.1', 'flavour': 'Mint'},

    # Mac OS X (source: https://en.wikipedia.org/wiki/MacOS#Release_history)
    {'name': 'cheetah', 'version': '10.0', 'full_name': 'Mac OS X 10.0 Cheetah', 'flavour': 'MacOS'},
    {'name': 'puma', 'version': '10.1', 'full_name': 'Mac OS X 10.1 Puma', 'flavour': 'MacOS'},
    {'name': 'jaguar', 'version': '10.2', 'full_name': 'Mac OS X 10.2 Jaguar', 'flavour': 'MacOS'},
    {'name': 'panther', 'version': '10.3', 'full_name': 'Mac OS X 10.3 Panther', 'flavour': 'MacOS'},
    {'name': 'tiger', 'version': '10.4', 'full_name': 'Mac OS X 10.4 Tiger', 'flavour': 'MacOS'},
    {'name': 'leopard', 'version': '10.5', 'full_name': 'Mac OS X 10.5 Leopard', 'flavour': 'MacOS'},
    {'name': 'snow leopard', 'version': '10.6', 'full_name': 'Mac OS X 10.6 Snow Leopard', 'flavour': 'MacOS'},
    {'name': 'lion', 'version': '10.7', 'full_name': 'Mac OS X 10.7 Lion', 'flavour': 'MacOS'},
    {'name': 'mountain lion', 'version': '10.8', 'full_name': 'OS X 10.8 Mountain Lion', 'flavour': 'MacOS'},
    {'name': 'mavericks', 'version': '10.9', 'full_name': 'OS X 10.9 Mavericks', 'flavour': 'MacOS'},
    {'name': 'yosemite', 'version': '10.10', 'full_name': 'OS X 10.10 Yosemite', 'flavour': 'MacOS'},
    {'name': 'el capitan', 'version': '10.11', 'full_name': 'OS X 10.11 El Capitan', 'flavour': 'MacOS'},
    {'name': 'sierra', 'version': '10.12', 'full_name': 'macOS 10.12 Sierra', 'flavour': 'MacOS'},
    {'name': 'high sierra', 'version': '10.13', 'full_name': 'macOS 10.13 High Sierra', 'flavour': 'MacOS'},
    {'name': 'mojave', 'version': '10.14', 'full_name': 'macOS 10.14 Mojave', 'flavour': 'MacOS'},
    {'name': 'catalina', 'version': '10.15', 'full_name': 'macOS 10.15 Catalina', 'flavour': 'MacOS'},
    {'name': 'big sur', 'version': '11.0', 'full_name': 'macOS 11 Big Sur', 'flavour': 'MacOS'},

    # Android (source: https://en.wikipedia.org/wiki/Android_version_history)
    {'name': 'cupcake', 'version': '1.5', 'full_name': 'Android 1.5 Cupcake', 'flavour': 'Android'},
    {'name': 'donut', 'version': '1.6', 'full_name': 'Android 1.6 Donut', 'flavour': 'Android'},
    {'name': 'eclair', 'version': '2.0', 'full_name': 'Android 2.0 Eclair', 'flavour': 'Android'},
    {'name': 'froyo', 'version': '2.2', 'full_name': 'Android 2.2 Froyo', 'flavour': 'Android'},
    {'name': 'gingerbread', 'version': '2.3', 'full_name': 'Android 2.3 Gingerbread', 'flavour': 'Android'},
    {'name': 'honeycomb', 'version': '3.0', 'full_name': 'Android 3.0 Honeycomb', 'flavour': 'Android'},
    {'name': 'ice cream sandwich', 'version': '4.0', 'full_name': 'Android 4.0 Ice Cream Sandwich', 'flavour': 'Android'},
    {'name': 'jelly bean', 'version': '4.1', 'full_name': 'Android 4.1 Jelly Bean', 'flavour': 'Android'},
    {'name': 'kitkat', 'version': '4.4', 'full_name': 'Android 4.4 KitKat', 'flavour': 'Android'},
    {'name': 'lollipop', 'version': '5.0', 'full_name': 'Android 5.0 Lollipop', 'flavour': 'Android'},
    {'name': 'marshmallow', 'version': '6.0', 'full_name': 'Android 6.0 Marshmallow', 'flavour': 'Android'},
    {'name': 'nougat', 'version': '7.0', 'full_name': 'Android 7.0 Nougat', 'flavour': 'Android'},
    {'name': 'oreo', 'version': '8.0', 'full_name': 'Android 8.0 Oreo', 'flavour': 'Android'},
    {'name': 'pie', 'version': '9', 'full_name': 'Android 9 Pie', 'flavour': 'Android'},
    {'name': 'android10', 'version': '10', 'full_name': 'Android 10', 'flavour': 'Android'},
    {'name': 'android11', 'version': '11', 'full_name': 'Android 11', 'flavour': 'Android'},
    {'name': 'android12', 'version': '12', 'full_name': 'Android 12', 'flavour': 'Android'},
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['distro']

package_data = \
{'': ['*']}

install_requires = \
['pylint>=2.7.2,<3.0.0']

entry_points = \
{'console_scripts': ['distro = distro.distro:main']}

setup_kwargs = {
    'name': 'distro-names',
    'version': '0.0.7',
    'description': 'Search for the names and versions of popular OS distributions',
    'long_description': '# distro-names\n\n`A way to search OS distribution names and versions`\n\n[![Build Status](https://travis-ci.org/igorlg/distro-names.svg?branch=master)](https://travis-ci.org/igorlg/distro-names)\n\nAren\'t you tired of searching things like "Which version is Ubuntu Trusty again", or "is Debian 8 _stretch_ or _wheezy_"? Well, I am... so I wrote this small python script to match OS versions with their names and vice versa.\n\nSimply use the CLI to search like this:\n\n```bash\n$ distro trusty\nFlavour:      Ubuntu\nFull Name:    Trusty Thar\nName:         trusty\nVersions:     14.04, 14.04.6\n```\n\nor use the version:\n```bash\n$ distro debian 7\nFlavour:     Debian\nName:        wheezy\nVersions:    7\n```\n\n# Install\n\nUsing Python pip:\n\n```bash\n$ pip install distro-names\n```\n\n# Contributing\n\nPlease feel free to contribute to this humble project - you can add distro names and versions to `distro/data.py` or improve the search capabilities in `distro/distro.py`.\n\nThis is a `pure Python` implementation, so no libs are required. If you\'d like to contribute, only PyTest and make are required...\n',
    'author': 'Igor Gentil',
    'author_email': 'igorlg@amazon.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/igorlg/distro-names',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
